// this implements a search function, for example in this page fragment:
/*
<div id="search">
   <div id="searchtext">
      Search:&nbsp;
   </div>
   <ul id="searchlist">
      <li> <input type="text" 
		  spellcheck="false"
		  id="searchfor" 
		  size="32"
		  placeholder="Type here your search terms and click Search!" />
	 <button id="clearbutton">Clear!</button>
	 <button id="searchbutton">Search!</button>
      </li>
   </ul>
</div>
<div id="result">
</div>
*/
// and here an example of a css:
// Note that many items are generated by the program
/*
a {
   text-decoration: none;
}
#there-is-more1:before {
   content:"...";
}
#there-is-more2:before {
   content:"Too many results, enter longer or more search terms...";
}
.resrow {
   padding-left : 60px; padding-right: 50px;
}
#reshr {   
   color:#383838; 
   background:#383838;
   height:1px;
   display:none;
}
#resheader {       
   color: #861509; 
   cursor: pointer;
}
#resheader:before {
   content: "Page title ↕"; 
}
#result {
   clear: both;
}
#resulthead {
   color: brown;
   font-style: italic;
}
#resulttable {
   padding-bottom: 10px; 
   padding-top: 10px;
}
table, th, td {
   border: 0px;
}
#resulttable a {
   color: brown;
}
#search {
   float: left;
   margin-top:-3px;
}
.searchchoice {
   color: purple;
   cursor: pointer;
}
#searchlist {
   list-style: none;
   float: left;
   margin-top: 0px;
}
#searchfor {
   color: #000000;
   margin-left: -20px;
   font-size: 16px;
}
#searchtext {
   color: brown;
   float:left;
   margin-left:20px;
   margin-top:3px;
}

#searchbutton {
   background-color: orange;
   color: black;
}
#clearbutton { 
   background-color: green;
   color: white;
}
#searchbutton, #clearbutton {
   display: inline-block;
   font-size: 16px;
   padding: 3px 3px;
   text-align: center;
   text-decoration: none;
   cursor: pointer;
   border-radius: 8px;
}
*/
/*
 * The program expects that the arrays PAGES and WORDS are present, here a short example:
 *
 *   const PAGES =[   // [url, title] 
 *   ['./index.html', 'RatrabbiT'],
 *   ['./contact/index.html', 'contact'],
 *   ['./contact/thanks/index.html', 'contact'],
 *   ['./contact/missing/index.html', 'contact'],
 *   ['./wp-admin/index.html', 'RatrabbiT'],
 *   ['./alracTTG/index.html', 'alracTTG'],
 *   ['./alracTTG/downloads/index.html', 'alractTTG-downloads'],
 *   ];
 *   
 *   const WORDS ={  // word: [indices of PAGES where word is present]
 *   'tournament' : [0, 4, 5, 26, 34],
 *   'useful' : [0, 4, 29, 33, 34, 45, 71, 76, 81],
 *   'web' : [0, 4, 5, 6, 8, 17, 24, 26, 27, 34, 37, 58, 71, 73, 77, 88, 94, 95],
 *   'website' : [0, 1, 4, 6, 9, 24, 25, 26, 33, 34, 67, 81, 85, 86, 88, 90, 91, 92, 94, 96],
 *   'welcome' : [0, 4, 18, 19, 21, 24, 29, 34, 58, 73],
 *   'wsnow' : [0, 4, 8, 9, 17, 25, 26, 34],
 *   };
 *
 *   For example: the word 'tournament' is present in PAGES[0], PAGES[4], PAGES[5], PAGES[26], PAGES[34]
 *
 */


// Here we go ...
//
$(document).ready(function(){

   if ( $("#checkforsearch").length == 0 ){
      return;
   }

   var choicemenu      = [];
   var sortorder       = true;
   var searched        = false;

   // we need a sorted array of the words in WORDS:
   //
   var words = Object.keys(WORDS).sort();

   var toptext = "";

   getitems();

   // create table based on result
   // result: array of indices in PAGES
   //
   function mktable(result){
      var table = $('<table id="resulttable"></table>');
      tablekop = $('<tr> <th id="resheader" class="resrow"></th></tr>');
      table.append(tablekop);
      for (var i = 0; i < result.length; i++){
	 var r = result[i];
	 var page = PAGES[r][0];
	 var title = PAGES[r][1];
	 // The ../ is for the wdcms situation to get a valid url
	 var url = '<a href="../'+page+'">'+title+'</a>';
	 table.append($('<tr><td class="resrow">'+url+'</td></tr>'))
      }
      return table;
   }

   // Handle the search terms, create a list of pages where the words
   // can be found
   //
   $("#searchbutton").click(function(){
      setitems();
      searchnorm();
      $(".choicelist").remove();
      var resdiv = $("#result")
      resdiv.html("");

      var result = [];
      var texts = $("#searchfor").val().trim().split(/\s+/);
      for (var textnum = 0; textnum < texts.length; textnum++){
	 var text = texts[textnum];
	 var res = WORDS[text];
	 if (false){ // in case you want the first hit as default, make it true ...
	    if (! res && toptext != ""){
	       var x = toptext.split(" ")[textnum];
	       if(x){
		  text = x;
		  texts[textnum] = text;
		  res = WORDS[text];
		  $("#searchfor").val(toptext);
	       }
	    }
	 }
	 if (textnum == 0){
	    result = res;
	    continue;
	 }
	 result = performIntersection(result,res);
      }

      if (result && result.length) {
	 var restext = result.length + ' results for: "'
	 if (result.length == 1){
	    restext = 'One result for: '
	 }
	 resdiv.append('<p id="resulthead">'+restext+texts.join(" ")+'"<br></p>');
	 result.sort(
	    function(a,b){
	       var r;
	       if ( PAGES[a][1].toLowerCase() > PAGES[b][1].toLowerCase()){
		  r = 1;
	       } else {
		  r = -1;
	       }
	       if (sortorder) {
		  return r;
	       } else {
		  return -r;
	       }
	    }
	 );
	 table = mktable(result);
	 resdiv.append(table);

	 $("#resheader").click(function(){
	    sortorder = !sortorder;
	    $("#searchbutton").click();
	 });
      } else {
	 if (texts.join(" ").trim()){
	    resdiv.append('<p id="resulthead">No results for: "'+texts.join(" ")+'"<br></p>');
	 }
      }
      var w = $("#search").width();
      resdiv.append($('<br><hr id="reshr" width="10" align="left">'));
      $("#reshr").width(w);
   });


   // generate a list of urls
   //
   function choicelist(){
      searchnorm();
      var searchfor = $("#searchfor").val().trim();
      if (searchfor == ""){
	 $(".choicelist").remove();
	 $("#result").html("");
	 return;
      }
      $("#result").html("");
      var texts = searchfor.split(/\s+/);

      $(".choicelist").remove();
      var found = 0;
      toptext = "";
      mkchoice(texts);
      for (var i = 0; i<choicemenu.length; i++){
	 row = choicemenu[i];
	 $("#searchlist").append('<li class="choicelist searchchoice">'+row[0].join(" ")+ ' ('+ row[1].length+')' + '</li>');
	 found += 1;
	 if (found == 1){
	    toptext = row[0].join(" ");
	 }
	 if (found > 24){
	    $("#searchlist").append('<li class="choicelist er-is-meer" id="there-is-more1"></li>');
	    $("#searchlist").append('<li class="choicelist er-is-meer" id="there-is-more2"></li>');
	    break;
	 }
      }
      $(".searchchoice").click(function(){
	 $("#searchfor").val($(this).text().replace(/ *\(.*/,''));  if(0)"x".replace(/\)/,'');
	 //                                                         to persuade vim that parentheses
	 //                                                         are ok...
	 $("#searchbutton").click();
      });
   }


   // produce result when one enters 'enter' in the search field
   //
   $("#searchfor").keyup(function(event){
      var keycode = (event.keyCode ? event.keyCode : event.which);
      if(keycode == '13'){
	 $("#searchbutton").click();
	 return;
      }
      choicelist();
   });

   $("#searchfor").click(function(){
      choicelist();
   });

   // clear some things
   //
   function clearall(){
      $("#searchfor").val("");
      $("#result").html("");
      $(".choicelist").remove();
      toptext = "";
      setitems();
   }

   $("#clearbutton").click(function(){
      clearall();
   });

   // This is for the case that we are going back to this page.
   // We want that the search result is displayed again.
   function restore()
   {
      if (!searched) {
	 if ($("#searchfor").val().trim() != "") {
	    $("#searchbutton").click();
	    searched = true;
	    return;
	 }
	 searched = true;
      }
   }
   window.onpageshow = function() {  
      // chromium does nothing here ...
      restore();
   }

   // chromium does arrive here after going back to this page ...
   restore();

   function clearitems(){
      try {
	 sessionStorage.clear();
	 sessionStorage.setItem("searchfor", "");
	 sessionStorage.setItem("sortorder", "true");
      } 
      catch(err){}
   }

   // input: m integer
   //        s string
   //        w array of strings
   // returns true s is a substring of w[m], starting at w[m][0]
   function valid(m,s,w) {
      var rc = false;
      if (m < w.length) {
	 if (w[m].indexOf(s) == 0) {
	    rc = true;
	 }
      }
      return rc;
   }

   // adds choices to choicemenu, based on strings
   // k: array of strings (normally the words to search for)
   // first: if true, create a new choicemenu
   // Note: this function is recursive, when called from outside
   // supply onlly the first parameter
   //
   function mkchoice(k,first=true){
      var maxadd = 100;               // add this many to  choicemenu
      var addings = 0;
      if (k.length == 0){
	 return;
      }
      m = binsearch(k[0],words,0,words.length);
      if (first){
	 first = false;
	 choicemenu = [];
	 addings = 0;
	 var n = m;
	 while (true){
	    if (!valid(n,k[0],words) || addings > maxadd){
	       break;
	    }
	    var w = words[n];
	    var intersect = WORDS[w].sort();
	    addings += 1;
	    choicemenu.push([[w],intersect]);
	    n += 1;
	 }
      } else {
	 var choicemenu_prev = choicemenu;
	 choicemenu = [];
	 addings  = 0;
	 for (i=0; i<choicemenu_prev.length; i++){
	    prev = choicemenu_prev[i];
	    var n = m;
	    while (true) {
	       if (!valid(n,k[0],words) || addings > maxadd) {
		  break;
	       }
	       var w = words[n];
	       intersect = performIntersection(WORDS[w],prev[1]);

	       if (intersect.length > 0) {
		  addings += 1;
		  keys = prev[0].slice();
		  keys.push(w);
		  choicemenu.push([keys,intersect]);
	       }
	       n += 1;
	    }
	 }
      }
      mkchoice(k.slice(1),first);
   }

   // normalize input terms
   //
   function searchnorm(){
      $("#searchfor").val(
	 $("#searchfor").val().toLowerCase().replace(/^\s+/,"").normalize("NFD").replace(/[\u0300-\u036f]/g, ""));
   }

   // perform intersection between to arrays, which are treated as sets, by the way.
   // Strange that javascript has a notion of 'set' but has no intersect function or operator.
   //
   function performIntersection(arr1, arr2) {
      // thanks to https://www.programiz.com/javascript/examples/intersection-two-arrays
      const setA = new Set(arr1);
      const setB = new Set(arr2);
      var intersectionResult = [];
      for (var i of setB) {
	 if (setA.has(i)) {
	    intersectionResult.push(i);
	 }
      }
      return intersectionResult;
   }

   // binary search
   // search for string in array of strings, return the index of the first element
   // that is larger or equal to the needle
   // s: string to search for
   // w: array of strings to search in
   // f: first index in w to use
   // l: last index in w to use
   //
   function binsearch(s,w,f,l){
      if (f == l) {
	 return f;
      }
      m = Math.floor((f+l)/2);
      if (s > w[m]) {
	 return binsearch(s,w,m+1,l);
      }
      else {
	 return binsearch(s,w,f,m);
      }
   }

   // place some items in sessioStorage
   //
   function setitems(){
      if (sortorder === undefined){
	 sortorder = true;
      };
      try {
	 sessionStorage.setItem("searchfor", $("#searchfor").val());
	 sessionStorage.setItem("sortorder", sortorder.toString());
      }
      catch(err){}
   }

   // retrieve some items from sessionStore
   //
   function getitems(){
      try {
	 searchfor = sessionStorage.getItem("searchfor");
	 var s = sessionStorage.getItem("sortorder");
	 if (!searchfor){
	    searchfor = "";
	 }
	 $("#searchfor").val(sessionStorage.getItem("searchfor"));
	 if (!s){
	    s = "true";
	 }
	 sortorder = (s == "true");
      } 
      catch(err){}
   }

});
